# Import local modules
from photoshop.api.colors.cmyk import CMYKColor
from photoshop.api.colors.gray import GrayColor
from photoshop.api.colors.hsb import HSBColor
from photoshop.api.colors.lab import LabColor
from photoshop.api.colors.rgb import RGBColor


__all__ = [
    CMYKColor.__name__,
    GrayColor.__name__,
    HSBColor.__name__,
    LabColor.__name__,
    RGBColor.__name__,
]



# Import local modules
from photoshop.api._core import Photoshop
from photoshop.api.enumerations import ColorReductionType
from photoshop.api.enumerations import DitherType


class ExportOptionsSaveForWeb(Photoshop):
    """Options for exporting Save For Web files."""

    object_name = "ExportOptionsSaveForWeb"

    def __init__(self):
        super().__init__()
        self.format = 13  # PNG
        self.PNG8 = False  # Sets it to PNG-24 bit

    @property
    def PNG8(self) -> bool:
        """If true, uses 8 bits. If false, uses 24 bits. Valid only when ‘format’ = PNG."""
        return self.app.PNG8

    @PNG8.setter
    def PNG8(self, value: bool):
        self.app.PNG8 = value

    @property
    def blur(self) -> float:
        """Applies blur to the image to reduce artifacts."""
        return self.app.blur

    @blur.setter
    def blur(self, value: float):
        self.app.blur = value

    @property
    def colorReduction(self) -> ColorReductionType:
        """The color reduction algorithm."""
        return self.app.colorReduction

    @colorReduction.setter
    def colorReduction(self, value: ColorReductionType):
        self.app.colorReduction = value

    @property
    def colors(self) -> int:
        """The number of colors in the palette."""
        return self.app.colors

    @colors.setter
    def colors(self, value: int):
        self.app.colors = value

    @property
    def dither(self) -> DitherType:
        """The type of dither to use."""
        return self.app.dither

    @dither.setter
    def dither(self, value: DitherType):
        self.app.dither = value

    @property
    def quality(self) -> int:
        """The quality of the output image, from 0 to 100."""
        return self.app.quality

    @quality.setter
    def quality(self, value: int):
        self.app.quality = value


class PNGSaveOptions(Photoshop):
    """Options for saving file as PNG."""

    object_name = "PNGSaveOptions"

    def __init__(self, interlaced: bool = False, compression: int = 6):
        super().__init__()
        self.interlaced = interlaced
        self.compression = compression

    @property
    def interlaced(self) -> bool:
        return self.app.interlaced

    @interlaced.setter
    def interlaced(self, value: bool):
        self.app.interlaced = value

    @property
    def compression(self) -> int:
        return self.app.compression

    @compression.setter
    def compression(self, value: int):
        self.app.compression = value



# Import third-party modules
from comtypes import ArgumentError

# Import local modules
from photoshop.api._core import Photoshop
from photoshop.api.errors import PhotoshopPythonAPIError
from photoshop.api.text_font import TextFont


class TextFonts(Photoshop):
    """An installed font."""

    def __init__(self, parent=None):
        super().__init__(parent=parent)

    def __iter__(self):
        for font in self.app:
            yield TextFont(font)

    def __getitem__(self, key: str):
        """Access a given TextFont using dictionary key lookup, must provide the postScriptName."""
        try:
            return TextFont(self.app[key])
        except ArgumentError:
            raise PhotoshopPythonAPIError(f'Could not find a font with postScriptName "{key}"')

    @property
    def _fonts(self):
        return [a for a in self.app]

    def __len__(self):
        return self.length

    @property
    def length(self):
        """The number pf elements in the collection."""
        return len(self._fonts)

    def getByName(self, name: str) -> TextFont:
        """Gets the font by the font name.

        Args:
            name: The name of the font.


        Returns:
            font instance.

        """
        for font in self.app:
            if font.name == name:
                return TextFont(font)
        raise PhotoshopPythonAPIError('Could not find a TextFont named "{name}"')



# https://theiviaxx.github.io/photoshop-docs/Photoshop/BatchOptions.html
# Import local modules
from photoshop.api._core import Photoshop


class BatchOptions(Photoshop):
    object_name = "BatchOptions"

    def __init__(self):
        super().__init__()

    @property
    def destination(self):
        """The type of destination for the processed files."""
        return self.app.destination

    @destination.setter
    def destination(self, value):
        self.app.destination = value

    @property
    def destinationFolder(self):
        """The folder location for the processed files. Valid only when ‘destination’ = folder."""
        return self.app.destinationFolder

    @destinationFolder.setter
    def destinationFolder(self, path):
        self.app.destinationFolder = path

    @property
    def errorFile(self):
        """The file in which to log errors encountered.
        To display errors on the screen and stop batch processing when errors occur, leave blank."""
        return self.app.errorFile

    @errorFile.setter
    def errorFile(self, file_path):
        self.app.errorFile = file_path

    @property
    def fileNaming(self) -> list:
        """A list of file naming options. Maximum: 6."""
        return self.app.fileNaming

    @fileNaming.setter
    def fileNaming(self, file_naming: list):
        self.app.fileNaming = file_naming

    @property
    def macintoshCompatible(self) -> bool:
        """If true, the final file names are Macintosh compatible."""
        return self.app.macintoshCompatible

    @macintoshCompatible.setter
    def macintoshCompatible(self, value: bool):
        self.app.macintoshCompatible = value

    @property
    def overrideOpen(self) -> bool:
        """If true, overrides action open commands."""
        return self.app.overrideOpen

    @overrideOpen.setter
    def overrideOpen(self, value: bool):
        self.app.overrideOpen = value

    @property
    def overrideSave(self) -> bool:
        """If true, overrides save as action steps with the specified destination."""
        return self.app.overrideSave

    @overrideSave.setter
    def overrideSave(self, value: bool):
        self.app.overrideSave = value

    @property
    def startingSerial(self) -> int:
        """The starting serial number to use in naming files."""
        return self.app.startingSerial

    @startingSerial.setter
    def startingSerial(self, value: int):
        self.app.startingSerial = value

    @property
    def suppressOpen(self) -> bool:
        """If true, suppresses file open options dialogs."""
        return self.app.suppressOpen

    @suppressOpen.setter
    def suppressOpen(self, value: bool):
        self.app.suppressOpen = value

    @property
    def suppressProfile(self) -> bool:
        """If true, suppresses color profile warnings."""
        return self.app.suppressProfile

    @suppressProfile.setter
    def suppressProfile(self, value: bool):
        self.app.suppressProfile = value

    @property
    def unixCompatible(self) -> bool:
        """If true, the final file names are Unix compatible."""
        return self.app.unixCompatible

    @unixCompatible.setter
    def unixCompatible(self, value: bool):
        self.app.unixCompatible = value

    @property
    def windowsCompatible(self) -> bool:
        """If true, the final file names are Windows compatible."""
        return self.app.windowsCompatible

    @windowsCompatible.setter
    def windowsCompatible(self, value: bool):
        self.app.windowsCompatible = value



# Import third-party modules
from comtypes import ArgumentError

# Import local modules
from photoshop.api._core import Photoshop
from photoshop.api._layerSet import LayerSet
from photoshop.api.errors import PhotoshopPythonAPIError


class LayerSets(Photoshop):
    """The layer sets collection in the document."""

    def __init__(self, parent):
        super().__init__(parent=parent)
        self._flag_as_method(
            "add",
            "item",
            "removeAll",
        )


    def __len__(self):
        return self.length

    def __iter__(self):
        for layer_set in self.app:
            yield layer_set

    def __getitem__(self, key: str):
        """Access a given LayerSet using dictionary key lookup."""
        try:
            return LayerSet(self.app[key])
        except ArgumentError:
            raise PhotoshopPythonAPIError(f'Could not find a LayerSet named "{key}"')

    @property
    def _layerSets(self):
        return list(self.app)

    @property
    def length(self) -> int:
        """Number of elements in the collection."""
        return len(self._layerSets)

    def add(self):
        return LayerSet(self.app.add())

    def item(self, index: int) -> LayerSet:
        return LayerSet(self.app.item(index))

    def removeAll(self):
        self.app.removeAll()

    def getByIndex(self, index: int):
        """Access LayerSet using list index lookup."""
        return LayerSet(self._layerSets[index])

    def getByName(self, name: str) -> LayerSet:
        """Get the first element in the collection with the provided name."""
        for layer in self.app:
            if name == layer.name:
                return LayerSet(layer)
        raise PhotoshopPythonAPIError(f'Could not find a LayerSet named "{name}"')



# Import local modules
from photoshop.api.save_options.bmp import BMPSaveOptions
from photoshop.api.save_options.eps import EPSSaveOptions
from photoshop.api.save_options.gif import GIFSaveOptions
from photoshop.api.save_options.jpg import JPEGSaveOptions
from photoshop.api.save_options.pdf import PDFSaveOptions
from photoshop.api.save_options.png import ExportOptionsSaveForWeb
from photoshop.api.save_options.png import PNGSaveOptions
from photoshop.api.save_options.psd import PhotoshopSaveOptions
from photoshop.api.save_options.tag import TargaSaveOptions
from photoshop.api.save_options.tif import TiffSaveOptions


__all__ = [
    BMPSaveOptions.__name__,
    EPSSaveOptions.__name__,
    GIFSaveOptions.__name__,
    JPEGSaveOptions.__name__,
    PDFSaveOptions.__name__,
    ExportOptionsSaveForWeb.__name__,
    PNGSaveOptions.__name__,
    PhotoshopSaveOptions.__name__,
    TargaSaveOptions.__name__,
    TiffSaveOptions.__name__,
]



# Import local modules
from photoshop.api._artlayer import ArtLayer
from photoshop.api._core import Photoshop
from photoshop.api.errors import PhotoshopPythonAPIError


# pylint: disable=too-many-public-methods
class Layers(Photoshop):
    """The layers collection in the document."""

    def __init__(self, parent):
        super().__init__(parent=parent)
        self._flag_as_method(
            "add",
            "item",
        )

    @property
    def _layers(self):
        return list(self.app)

    def __len__(self):
        return self.length

    def __getitem__(self, key):
        item = self._layers[key]
        return ArtLayer(item)

    @property
    def length(self):
        return len(self._layers)

    def removeAll(self):
        """Deletes all elements."""
        for layer in self.app:
            ArtLayer(layer).remove()

    def item(self, index):
        return ArtLayer(self.app.item(index))

    def __iter__(self):
        for layer in self._layers:
            yield ArtLayer(layer)

    def getByName(self, name: str) -> ArtLayer:
        """Get the first element in the collection with the provided name."""
        for layer in self.app:
            if layer.name == name:
                return ArtLayer(layer)
        raise PhotoshopPythonAPIError("X")



"""This object provides an array-style mechanism for storing data.

It can be used for low-level access info Photoshop.


"""
# Import local modules
from photoshop.api._core import Photoshop


class ActionList(Photoshop):
    """The list of commands that comprise an Action.

    (such as an Action created using the Actions palette in the Adobe Photoshop application).
    The action list object is part of the Action Manager functionality.
    For details on using the Action Manager, see the Photoshop Scripting Guide.

    """

    object_name = "ActionList"

    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self._flag_as_method(
            "getBoolean",
            "getClass",
            "getData",
            "getDouble",
            "getEnumerationType",
            "getEnumerationValue",
            "getInteger",
            "getLargeInteger",
            "getList",
            "getObjectType",
        )

    @property
    def count(self):
        return self.app.count

    def getBoolean(self, index):
        return self.app.getBoolean(index)

    def getClass(self, index):
        return self.app.getClass(index)

    def getData(self, index):
        return self.app.getData(index)

    def getDouble(self, index):
        return self.app.getDouble(index)

    def getEnumerationType(self, index):
        return self.app.getEnumerationType(index)

    def getEnumerationValue(self, index):
        return self.app.getEnumerationValue(index)

    def getInteger(self, index):
        return self.app.getInteger(index)

    def getLargeInteger(self, index):
        return self.app.getLargeInteger(index)

    def getList(self, index):
        return self.app.getList(index)

    def getObjectType(self, index):
        return self.app.getObjectType(index)



# Import third-party modules
from comtypes import ArgumentError

# Import local modules
from photoshop.api._artlayer import ArtLayer
from photoshop.api._core import Photoshop
from photoshop.api.errors import PhotoshopPythonAPIError


# pylint: disable=too-many-public-methods
class ArtLayers(Photoshop):
    """The collection of art layer objects in the document."""

    def __init__(self, parent):
        super().__init__(parent=parent)
        self._flag_as_method(
            "add",
        )

    @property
    def _layers(self):
        return list(self.app)

    def __len__(self):
        return self.length

    def __iter__(self):
        for layer in self.app:
            yield layer

    def __getitem__(self, key: str):
        """Access a given ArtLayer using dictionary key lookup."""
        try:
            return ArtLayer(self.app[key])
        except ArgumentError:
            raise PhotoshopPythonAPIError(f'Could not find an artLayer named "{key}"')

    @property
    def length(self):
        return len(self._layers)

    @property
    def parent(self):
        return self.app.parent

    @property
    def typename(self):
        return self.app.typename

    def add(self):
        """Adds an element."""
        return ArtLayer(self.app.add())

    def getByIndex(self, index: int):
        """Access ArtLayer using list index lookup."""
        return ArtLayer(self._layers[index])

    def getByName(self, name: str) -> ArtLayer:
        """Get the first element in the collection with the provided name.

        Raises:
            PhotoshopPythonAPIError: Could not find a artLayer.
        """
        for layer in self.app:
            if layer.name == name:
                return ArtLayer(layer)
        raise PhotoshopPythonAPIError(f'Could not find an artLayer named "{name}"')

    def removeAll(self):
        """Deletes all elements."""
        for layer in self.app:
            ArtLayer(layer).remove()
